﻿
using SuValue.BLL;
using SuValue.Database.Entity;
using SuValue.Entity;
using SuValue.Entity.Enum;
using SuValue.Entity.Responese;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;

namespace SuValue.API.Controllers
{

    /// <summary>
    /// 用户信息
    /// </summary>
    public class UserController : BaseController
    {
        UserBLL bll = new UserBLL();

        /// <summary>
        /// 将验证码发送至用户手机
        /// </summary>
        /// <param name="model">校验码</param>
        /// <param name="mobile">手机号</param>
        /// <param name="type">验证方式：1是短信 2是语音</param>
        /// <returns></returns>
        [HttpGet]
        public EntityResult Message([FromUri]EntityBase model, string mobile, SMSType type, MessageType messagetype)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                SMSBLL sbll = new SMSBLL();//短信验证码

                EntityResult result = sbll.Mesage(mobile, type, messagetype);
                //返回一个状态，用于判断是否成功
                //如果成功，页面就提示“验证码获取成功”
                return result;
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 手机验证
        /// </summary>
        /// <param name="model"></param>
        /// <param name="mobile"></param>
        /// <param name="code"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        [HttpGet]
        public EntityResult CheckMessage([FromUri]EntityBase model, string mobile, string code, SMSType type, MessageType messagetype)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                SMSBLL sbll = new SMSBLL();//短信验证码

                EntityResult result = sbll.CheckMessage(mobile, code, type, messagetype);

                //返回一个状态，用于判断是否成功

                //如果成功，页面就提示“验证码获取成功”
                return result;
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 根据用户ID查询用户信息
        /// </summary>
        /// <param name="model">校验码</param>
        /// <param name="userid">用户ID</param>
        /// <returns>此接口用于个人信息查询</returns>
        [HttpGet]
        public EntityResult GetUserList([FromUri]EntityBase model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                sys_user list = bll.GetUserInfo(Convert.ToInt32(model.userid));

                if (list == null) return new EntityResult(ResultType.Fail, "用户信息不存在");

                EntityUser lists = new EntityUser(list);

                return new EntityResult(ResultType.Success, "", lists);
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }


        /// <summary>
        /// 修改用户信息
        /// </summary>
        /// <param name="model">校验码</param>
        /// <param name="UserName">用户名称</param>
        /// <param name="HeadPortrait">头像地址</param>
        /// <param name="Mobile">手机号</param>
        /// <param name="Mail">邮箱</param>
        /// <param name="CorporateName">公司名称</param>
        /// <param name="Department">部门</param>
        /// <param name="Duties">职务</param>
        /// <param name="ID">用户ID</param>
        /// <param name="LandingCode">绑定微信号</param>
        /// <param name="LandingVoucher">登陆凭证</param>
        /// <returns></returns>
        [HttpPost]
        public EntityResult PostUpdate([FromBody] SuValue.Entity.Request.EntityUser model)
        {

            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                EntityResult result = bll.Update(model);

                return result;
            }
            catch (Exception ex)
            {

                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 手机号登陆
        /// </summary>
        /// <param name="model">校验码</param>
        /// <param name="mobile">手机号码</param>
        /// <param name="code">验证码</param>
        [HttpGet]
        public EntityResult Login([FromUri]EntityBase model, string mobile, string code)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);
                EntityResult result = bll.Login(mobile, code, model.token);
                return result;
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }
    }
}
