﻿
using SuValue.DAL;
using SuValue.Database.Entity;
using SuValue.Entity;
using SuValue.Entity.Enum;
using SuValue.Helpers.Message;
using System;
using System.Collections.Generic;
using System.Net;

namespace SuValue.BLL
{
    /// <summary>
    /// 短信
    /// </summary>
    public class SMSBLL
    {

        /// <summary>
        /// 获取验证码
        /// </summary>
        /// <param name="mobile">手机号</param>
        /// <param name="type">验证方式：1是短信  2是语音</param>
        /// <returns></returns>
        public EntityResult Mesage(string mobile, SMSType smstype, MessageType messagetype)
        {

            EntityResult result = new EntityResult(ResultType.Fail, "");

            if (!MessageHelper.IsMobilePhone(mobile)) return new EntityResult(ResultType.Fail, "手机号格式错误");

            using (sms_mtDAL dal = new sms_mtDAL())
            {
                //判断是否是用户

                sys_userDAL udal = new sys_userDAL(dal._db);

                sys_user user = udal.GetMobileLoginInfo(mobile);

                if (messagetype == MessageType.Login && user == null) return new EntityResult(ResultType.Fail, "对不起，您输入的手机号码不是系统用户");



                //生成随机数
                Random rad = new Random();

                int mobile_code = rad.Next(1000, 10000);
                //SMSType 数据库存储 ，所有消息都为1
                sms_mt mol = new sms_mt() { CreateDate = DateTime.Now, CreateUser = user == null ? 0 : user.ID, Message = mobile_code.ToString(), Mobile = mobile, SMSType = (int)SMSType.Message, MessageType = (int)messagetype };

                sms_mtDAL mdal = new sms_mtDAL(dal._db);

                result = mdal.Insert(mol);

                if (result.state == ResultType.Success)
                {
                    //如果保存成功
                    //调用发送验证码的接口

                    //短信验证
                    HttpWebResponse myResponse = MessageHelper.Message(mobile, mobile_code.ToString(), (int)smstype);

                    if (myResponse.StatusCode == HttpStatusCode.OK)
                    {

                        return new EntityResult(ResultType.Success, "验证码发送成功");
                    }
                    else
                    {
                        //访问失败
                        return new EntityResult(ResultType.Fail, "验证码获取失败");
                    }
                }

                return result;
            }
        }

        /// <summary>
        /// 校验验证码是否正确
        /// </summary>
        /// <param name="mobile">手机号码</param>
        /// <param name="type">验证码类型</param>
        /// <returns></returns>
        public EntityResult CheckMessage(string mobile, string code, SMSType smstype, MessageType messagetype)
        {
            EntityResult result = new EntityResult(ResultType.Success, "");

            if (!MessageHelper.IsMobilePhone(mobile)) return new EntityResult(ResultType.Fail, "手机号格式错误");

            //判断是否是用户
            using (sms_mtDAL dal = new sms_mtDAL())
            {
                sys_userDAL udal = new sys_userDAL(dal._db);

                sys_user user = udal.GetMobileLoginInfo(mobile);

                if (messagetype == MessageType.Login && user == null) return new EntityResult(ResultType.Fail, "对不起，您输入的手机号码不是系统用户");

                if (messagetype == MessageType.Modify && user != null) return new EntityResult(ResultType.Fail, "对不起，您输入的手机号码已经是系统用户");

                sms_mt mt = dal.GetSendMessage(mobile, code, smstype, messagetype);

                if (mt == null) return new EntityResult(ResultType.Fail, "验证码错误");

                return result;
            }
        }


    }
}
