﻿using SuValue.DAL;
using SuValue.Database;
using SuValue.Entity;
using SuValue.Helpers;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SuValue.BLL
{
    public class TaskBLL
    {


        private string sqlserver_connectionString = ConfigurationManager.ConnectionStrings["sqlserver_connectionString"].ToString();

        private string mysql_connectionString = ConfigurationManager.ConnectionStrings["mysql_connectionString"].ToString();

        /// <summary>
        /// 执行任务
        /// </summary>
        /// <param name="id">任务ID</param>
        /// <param name="hosid">医院ID，当未填写医院ID执行全部医院</param>
        /// <returns></returns>
        public EntityResult ExecTask(int id, int? hosid, DateTime begindate, DateTime enddate)
        {
            //获取所有医院清单
            using (sys_taskDAL dal = new sys_taskDAL())
            {

                sys_hospitalDAL hdal = new sys_hospitalDAL(dal._db);

                List<sys_hospital> hospitallist = hdal.GetValidHospital();
                if (hosid != null) //执行单家医院
                {
                    hospitallist = hospitallist.Where(o => o.ID == hosid).ToList();
                }

                //获取任务需要执行的所有脚本信息

                sys_task_configDAL tdal = new sys_task_configDAL(dal._db);

                List<sys_task_config> configlist = tdal.GetTaksConfigList(id);

                sys_logDAL logdal = new sys_logDAL(dal._db);

                sys_taskDAL taskdal = new sys_taskDAL(dal._db);

                sys_task task = taskdal.GetObject(id);

                if (task == null) return new EntityResult(ResultType.Fail, "任务ID不存在");

                foreach (var item in hospitallist) //开始执行
                {
                    try
                    {

                        if (string.IsNullOrEmpty(item.database)) continue;

                        logdal.Insert(new sys_log { createdate = DateTime.Now, Log = string.Format("医院编码：{0}医院名称： {1} 开始执行！", item.ID, item.HosName) });

                        ExecSql(task, item, configlist, begindate, enddate);

                        logdal.Insert(new sys_log { createdate = DateTime.Now, Log = string.Format("医院编码：{0}医院名称： {1} 执行完毕！", item.ID, item.HosName) });

                    }
                    catch (Exception ex)
                    {
                        continue;
                    }
                }

                return new EntityResult(ResultType.Success);
            }

        }

        private void ExecSql(sys_task task, sys_hospital hospital, List<sys_task_config> configlist, DateTime begindate, DateTime enddate)
        {
            //生成临时表

            string sqlserver_conn = sqlserver_connectionString.Replace("{server}", hospital.DataSource).Replace("{database}", hospital.database).Replace("{login}", hospital.login).Replace("{password}", hospital.password);

            string mysql_conn = mysql_connectionString.Replace("{server}", task.server).Replace("{database}", task.database).Replace("{login}", task.login).Replace("{password}", task.password);

            DbUtility db = new DbUtility(sqlserver_conn, Database.Enum.DbProviderType.SqlServer);

            var source156 = hospital.DataSource == "192.168.18.156" ? "" : "[192.168.18.156].";

            Initialization(db, task);

            foreach (var item in configlist)
            {
                try
                {

                        DbUtility mysqldb = new DbUtility(mysql_conn, Database.Enum.DbProviderType.MySql);

                    CreateTable(db, item, source156, hospital, begindate, enddate);

                    //导出CSV文件
                    CreateCSV(item, hospital.DataSource, hospital.database, hospital.login, hospital.password);

                    UpdaloadData(mysqldb, item, hospital, begindate, enddate);

                    //删除sql server数据表
                    DeleteTable(db, item.table_name);

                }
                catch (Exception ex)
                {
                    new sys_logDAL().Insert(new sys_log { begindate = DateTime.Now, createdate = DateTime.Now, enddate = DateTime.Now, Log = string.Format("导数医院：{0}，医院编码{1}，导数表{2}", hospital.HosName, hospital.ID, item.table_name), error_message = ex.ToString() });

                    continue;
                }
            }

            DropTable(db, task);

        }
        private void Initialization(DbUtility db, sys_task task)
        {
            if (string.IsNullOrEmpty(task.begin_sql)) return;

            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = "开始创建常用临时表" };

                try
                {
                    int result = db.ExecuteNonQuerySql(task.begin_sql);

                    log.Log += ",返回影响行数" + result;

                    log.enddate = DateTime.Now;
                }
                catch (Exception ex)
                {
                    log.error_message = ex.ToString();
                }

                logdal.Insert(log);
            }
        }

        private void DropTable(DbUtility db, sys_task task)
        {
            if (string.IsNullOrEmpty(task.end_sql)) return;

            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = "执行完成，清理临时表" };

                try
                {
                    int result = db.ExecuteNonQuerySql(task.end_sql);

                    log.Log += ",返回影响行数" + result;

                    log.enddate = DateTime.Now;
                }
                catch (Exception ex)
                {
                    log.error_message = ex.ToString();
                }

                logdal.Insert(log);
            }
        }

        private void CreateTable(DbUtility db, sys_task_config item, string source156, sys_hospital hospital, DateTime begindate, DateTime enddate)
        {
            string sql = "IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID('" + item.table_name + "') AND type in ('U')) BEGIN DROP TABLE " + item.table_name + " END; " + item.exec_sql.Replace("开始日期", begindate.ToShortDateString()).Replace("结束日期", enddate.ToShortDateString())
        .Replace("医院编码", hospital.ID.ToString()).Replace("服务器156地址", source156).Replace("表名称", item.table_name);

            int result = db.ExecuteNonQuerySql(sql);

            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = string.Format("开始生成SQLSERVR，{0}表数据", item.table_name) };

                log.Log += ",执行sql语句" + sql;

                log.Log += ",返回影响行数" + result;

                log.enddate = DateTime.Now;

                logdal.Insert(log);
            }
        }

        private void CreateCSV(sys_task_config table, string ip, string database, string login, string password)
        {
            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = string.Format("开始导出CSV文件，{0}表数据", table.table_name) };
                Process p = new Process();

                string url = System.AppDomain.CurrentDomain.BaseDirectory + "Data\\" + database + "\\";

                FileHelper.CreateDirectory(url);

                string export_sql = table.export_sql.Replace("数据库名称", database).Replace("表名称", table.table_name).Replace("服务器地址", ip).Replace("账号", login).Replace("密码", password).Replace("传输地址", url + table.table_name + ".csv");

                //table.export_sql = table.export_sql.Replace("数据库名称", database);
                //table.export_sql = table.export_sql.Replace("表名称", table.table_name);
                //table.export_sql = table.export_sql.Replace("服务器地址", ip);
                //table.export_sql = table.export_sql.Replace("账号", login);
                //table.export_sql = table.export_sql.Replace("密码", password);
                //table.export_sql = table.export_sql.Replace("传输地址", url + table.table_name + ".csv");

                //设置要启动的应用程序
                p.StartInfo.FileName = "cmd.exe";
                //是否使用操作系统shell启动
                p.StartInfo.UseShellExecute = false;
                // 接受来自调用程序的输入信息
                p.StartInfo.RedirectStandardInput = true;
                //输出信息
                p.StartInfo.RedirectStandardOutput = true;
                // 输出错误
                p.StartInfo.RedirectStandardError = true;
                //不显示程序窗口
                p.StartInfo.CreateNoWindow = false;
                //启动程序
                p.Start();

                //向cmd窗口发送输入信息
                p.StandardInput.WriteLine(@export_sql + "&exit");

                p.StandardInput.AutoFlush = true;

                //获取输出信息
                string strOuput = p.StandardOutput.ReadToEnd();
                //等待程序执行完退出进程
                p.WaitForExit();

                p.Close();

                log.Log += "结果信息：" + strOuput;

                log.enddate = DateTime.Now;


                logdal.Insert(log);
            }


        }

        /// <summary>
        /// 删除临时表
        /// </summary>
        /// <param name="db"></param>
        /// <param name="tablename"></param>
        private void DeleteTable(DbUtility db, string tablename)
        {
            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = string.Format("开始删除SQLSERVR，{0}表数据", tablename) };


                string sql = "drop table " + tablename + " ;";

                log.Log += ",执行sql语句" + sql;

                int result = db.ExecuteNonQuerySql(sql);

                log.Log += ",返回影响行数" + result;

                log.enddate = DateTime.Now;

                logdal.Insert(log);
            }
        }


        private void UpdaloadData(DbUtility db, sys_task_config item, sys_hospital hospital, DateTime begindate, DateTime enddate)
        {
            //删除服务器上面数据

            string delete_sql = item.delete_sql.Replace("医院编码", hospital.ID.ToString()).Replace("开始日期", begindate.ToString("yyyy-MM-dd")).Replace("结束日期", enddate.ToString("yyyy-MM-dd")).Replace("表名称", item.table_name);

            string upload_sql = item.upload_sql.Replace("路径", System.AppDomain.CurrentDomain.BaseDirectory.Replace("\\", "/") + @"Data/" + hospital.database + "/" + item.table_name + ".csv").Replace("表名称", item.table_name);

            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = string.Format("开始删除mysql数据库中{0}表数据", item.table_name) };


                log.Log += ",执行sql语句" + delete_sql;

                int result = db.ExecuteNonQuerySql(delete_sql);

                log.Log += ",返回影响行数" + result;

                log.enddate = DateTime.Now;


                logdal.Insert(log);
            }

            //导入数据

            using (sys_logDAL logdal = new sys_logDAL())
            {
                sys_log log = new sys_log { createdate = DateTime.Now, begindate = DateTime.Now, Log = string.Format("开始导入mysql数据库中{0}表数据", item.table_name) };

                log.Log += ",执行sql语句" + upload_sql;

                int result = db.ExecuteNonQuerySql(upload_sql);

                log.Log += ",返回影响行数" + result;

                log.enddate = DateTime.Now;


                logdal.Insert(log);
            }

        }
    }
}