﻿
using SuValue.DAL;
using SuValue.Database;
using SuValue.Database.Entity;
using SuValue.Entity;
using SuValue.Entity.Enum;
using SuValue.Entity.Responese;
using SuValue.Helpers.Message;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;

namespace SuValue.BLL
{
    /// <summary>
    /// 用户
    /// </summary>
    public class UserBLL
    {



        /// <summary>
        /// Login
        /// </summary>
        /// <param name="mobile">手机号</param>
        /// <param name="code">验证码</param>
        /// <returns></returns>
        public EntityResult Login(string mobile, string code, string openkey)
        {

            EntityResult result = new EntityResult(ResultType.Success, "");

            if (!MessageHelper.IsMobilePhone(mobile))
            {
                return new EntityResult(ResultType.Fail, "手机号格式错误");
            }

            if (string.IsNullOrWhiteSpace(mobile) || string.IsNullOrWhiteSpace(code))
            {
                return new EntityResult(ResultType.Fail, "验证或手机号不能为空！");
            }
            using (sys_userDAL dal = new sys_userDAL())
            {
                sms_mtDAL sdal = new sms_mtDAL(dal._db);

                sms_mt mt = sdal.GetSendMessage(mobile, code, MessageType.Login);

                if (mt == null)//获取验证码没有保存成功
                {
                    return new EntityResult(ResultType.Fail, "验证码或手机号输入有误！");
                }


                //从原先1天改成60秒内有效
                if (mt.CreateDate.AddSeconds(60) <= DateTime.Now)
                {
                    return new EntityResult(ResultType.Fail, "验证码已经失效，请重新获取！");//验证码还要提示
                }

                List<sys_user_project> uplist = new sys_user_projectDAL(dal._db).GetObjects(" where userid = " + mt.CreateUser + " and projectid in (select id from sys_project where open_key = '" + openkey + "') ");

                if (uplist == null) return new EntityResult(ResultType.Fail, "登陆失败，此用户缺少与项目之前关联信息");



                //获取用户信息

                sys_user user = dal.GetObject(mt.CreateUser);

                if (user == null) return new EntityResult(ResultType.Fail, "用户信息不存在请联系管理员");

                EntityUser lists = new EntityUser(user);

                //返回一个list集合
                return new EntityResult(ResultType.Success, "", lists);
            }

        }



        /// <summary>
        /// 获取用户信息
        /// </summary>
        /// <param name="userid"></param>
        /// <returns></returns>
        public sys_user GetUserInfo(int userid)
        {
            using (sys_userDAL dal = new sys_userDAL())
            {
                //关联用户登陆表取微信号
                sys_user obj = dal.GetObject(userid);

                return obj;
            }

        }


        /// <summary>
        /// 修改用户信息
        /// </summary>
        /// <param name="mol">实体类</param>
        /// <returns></returns>
        public EntityResult Update(SuValue.Entity.Request.EntityUser model)
        {
            if (model == null) return new EntityResult(ResultType.Fail, "修改用户信息不存在");
            if (string.IsNullOrWhiteSpace(model.UserName) || string.IsNullOrWhiteSpace(model.Mobile)) return new EntityResult(ResultType.Fail, "姓名与手机号不能为空！");
            if (!MessageHelper.IsMobilePhone(model.Mobile)) return new EntityResult(ResultType.Fail, "手机号格式错误！");

            sys_user mol = GetUserInfo(model.ID);

            #region 赋值区

            mol.CorporateName = model.CorporateName;
            mol.Department = model.Department;
            mol.Duties = model.Duties;
            mol.HeadPortrait = model.HeadPortrait;
            mol.Mail = model.Mail;
            mol.Mobile = model.Mobile;
            mol.UserName = model.UserName;
            mol.Updateuser = model.ID;
            mol.UpdateDate = DateTime.Now;
            mol.Perfect = 1;

            #endregion

            //获取手机账号信息

            EntityResult result = new EntityResult(ResultType.Success);

            using (sys_userDAL dal = new sys_userDAL())
            {
                if (!string.IsNullOrEmpty(model.Mobile)) //如果手机号码为空，将不补全
                {

                    sys_logininfoDAL ldal = new sys_logininfoDAL(dal._db);

                    sys_logininfo mobileLoginInfo = ldal.GetUserLoginInfo(model.ID).Where(o => o.Type == (int)LoginInfoType.Mobile).FirstOrDefault();

                    if (mobileLoginInfo == null) //需要新增用户手机登陆账号信息
                    {
                        result = ldal.Insert(new sys_logininfo { CreateDate = DateTime.Now, LoginName = model.Mobile, Type = (int)LoginInfoType.Mobile, UserID = model.ID });
                    }
                    else if (mobileLoginInfo.LoginName != model.Mobile && !string.IsNullOrEmpty(model.MobileCode))
                    {
                        sms_mtDAL sdal = new sms_mtDAL(dal._db);

                        sms_mt mt = sdal.GetSendMessage(model.Mobile, model.MobileCode, MessageType.Modify);

                        if (mt == null) return new EntityResult(ResultType.Fail, "手机验证码错误");

                        mobileLoginInfo.LoginName = model.Mobile;

                        ldal.Update(mobileLoginInfo);
                    }
                }

                if (result.state != ResultType.Success) return result;

                result = dal.Update(mol);

                return result;
            }
        }



    }
}
