﻿
using SuValue.Database;
using SuValue.Database.Entity;
using SuValue.Entity;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SuValue.DAL
{
    public class DemoDAL : IDisposable
    {
        public DbUtility _db { get; set; }

        public void Dispose()
        {
            _db.CloseConnection();
        }

        #region 构造函数

        public DemoDAL()
        {
            _db = new DbUtility();

            _db.OpenConnection();
        }

        public DemoDAL(DbUtility db)
        {
            this._db = db;
        }

        #endregion

        #region 增删方法


        /// <summary>
        /// 删除对象
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public EntityResult Delete(int id)
        {
            string sql = _db.ObjectToDeleteSql<sys_user>(new sys_user { ID = id });

            int result = _db.ExecuteNonQuerySql(sql);

            if (result > 0) return new EntityResult(ResultType.Success);

            else return new EntityResult(ResultType.Error);

        }


        /// <summary>
        /// 添加对象
        /// </summary>
        /// <param name="t">添加对象</param>
        /// <returns>默认添加成功后，将自增主键放在Data中，根据自己的类型自行转换</returns>
        public EntityResult Insert(sys_user t)
        {
            string sql = _db.ObjectToInsertSql<sys_user>(t);

            int result = _db.ExecuteNonQuerySql(sql);

            int id = Convert.ToInt32(_db.ExecuteScalarSql(" select LAST_INSERT_ID(); "));

            t.ID = id;

            if (result > 0) return new EntityResult(ResultType.Success, string.Empty, id);

            else return new EntityResult(ResultType.Error);
        }

        /// <summary>
        /// 修改对象
        /// </summary>
        /// <param name="t">默认根据主键作为条件</param>
        /// <returns></returns>
        public EntityResult Update(sys_user t)
        {
            string sql = _db.ObjectToUpdateSql<sys_user>(t);

            int result = _db.ExecuteNonQuerySql(sql);

            if (result > 0) return new EntityResult(ResultType.Success);

            else return new EntityResult(ResultType.Error);
        }


        #endregion

        #region 返回单个对象

        /// <summary>
        /// 获取对象
        /// </summary>
        /// <returns>返回对象</returns>
        protected sys_user GetObject()
        {
            sys_user obj = _db.QueryForObjectSql<sys_user>(new sys_user());

            return obj;
        }

        /// <summary>
        /// 获取对象
        /// </summary>
        /// <param name="where">查询条件</param>
        /// <returns>返回对象</returns>
        protected sys_user GetObject(string where)
        {
            sys_user obj = _db.QueryForObjectSql<sys_user>(new sys_user(), where);

            return obj;
        }

        /// <summary>
        /// 获取对象
        /// </summary>
        /// <param name="where">查询条件</param>
        /// <param name="orderby">排序</param>
        /// <returns>返回对象</returns>
        protected sys_user GetObject(string where, string orderby)
        {
            sys_user obj = _db.QueryForObjectSql<sys_user>(new sys_user(), where, orderby);

            return obj;
        }

        #endregion

        #region 返回集合

        /// <summary>
        /// 获取对象集合
        /// </summary>
        /// <returns>返回象集合</returns>
        protected List<sys_user> GetObjects()
        {
            List<sys_user> obj = _db.QueryForListSql<sys_user>(new sys_user());

            return obj;
        }

        /// <summary>
        /// 获取对象集合
        /// </summary>
        /// <param name="where">查询条件</param>
        /// <returns>返回象集合</returns>
        protected List<sys_user> GetObjects(string where)
        {
            List<sys_user> obj = _db.QueryForListSql<sys_user>(new sys_user(), where);

            return obj;
        }

        /// <summary>
        /// 获取对象集合
        /// </summary>
        /// <param name="where">查询条件</param>
        /// <param name="orderby">排序</param>
        /// <returns>返回象集合</returns>
        protected List<sys_user> GetObjects(string where, string orderby)
        {
            List<sys_user> obj = _db.QueryForListSql<sys_user>(new sys_user(), where, orderby);

            return obj;
        }

        #endregion

        #region 返回分页集合

        /// <summary>
        /// 分页获取对象
        /// </summary>
        /// <param name="pageindex">当前页</param>
        /// <param name="pagesize">每页条数</param>
        /// <returns>返回分页集合</returns>
        protected Page<sys_user> GetObjects(int pageindex, int pagesize)
        {
            Page<sys_user> obj = _db.QueryForListSql<sys_user>(new sys_user(), pageindex, pagesize);

            return obj;
        }

        /// <summary>
        /// 分页获取对象
        /// </summary>
        /// <param name="pageindex">当前页</param>
        /// <param name="pagesize">每页条数</param>
        /// <param name="where">查询条件</param>
        /// <returns>返回分页集合</returns>
        protected Page<sys_user> GetObjects(int pageindex, int pagesize, string where)
        {
            Page<sys_user> obj = _db.QueryForListSql<sys_user>(new sys_user(), pageindex, pagesize, where);

            return obj;
        }

        /// <summary>
        /// 分页获取对象
        /// </summary>
        /// <param name="pageindex">当前页</param>
        /// <param name="pagesize">每页条数</param>
        /// <param name="where">查询条件</param>
        /// <param name="orderby">排序</param>
        /// <returns>返回分页集合</returns>
        protected Page<sys_user> GetObjects(int pageindex, int pagesize, string where, string orderby)
        {
            Page<sys_user> obj = _db.QueryForListSql<sys_user>(new sys_user(), pageindex, pagesize, where, orderby);

            return obj;
        }

        #endregion
    }
}
