﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SuValue.Database
{
    /// <summary>
    /// 自定义属性，用于指示如何从DataTable或者DbDataReader中读取类的属性值
    /// </summary>
    public class ColumnAttribute : Attribute
    {

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="isFilter">是否过滤</param>
        public ColumnAttribute(bool isFilter)
            : this("", isFilter, false)
        {

        }

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="columnName">属性对应的字段</param>
        public ColumnAttribute(string columnName)
            : this(columnName, false, false)
        {

        }

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="columnName">类属性对应的列名</param>
        /// <param name="isFilter">是否过滤</param>
        public ColumnAttribute(string columnName, bool isFilter)
            : this(columnName, isFilter, false)
        {
        }

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="columnName">类属性对应的列名</param>
        /// <param name="isFilter">是否过滤</param>
        /// <param name="isPrimaryKey">是否是主键</param>
        public ColumnAttribute(string columnName, bool isFilter, bool isPrimaryKey)
        {
            ColumnName = columnName;

            IsFilter = isFilter;

            IsPrimaryKey = isPrimaryKey;
        }

        /// <summary>
        /// 属性对应的字段名称
        /// </summary>
        public string ColumnName { get; set; }

        /// <summary>
        /// 属性是否参加拼写SQL语句
        /// </summary>
        public bool IsFilter { get; set; }

        /// <summary>
        /// 属性是否主键
        /// </summary>
        public bool IsPrimaryKey { get; set; }

        /// <summary>
        /// 属性值
        /// </summary>
        public object Value { get; set; }

        /// <summary>
        /// 字段数据类型
        /// </summary>
        public DbType? dbType { get; set; }


    }
}
