﻿using SuValue.Entity;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SuValue.Entity
{
    /// <summary>
    /// 所有业务层非返回集合或者对象的方法必须返回此类
    /// </summary>
    public class EntityResult
    {
        public EntityResult(ResultType state) : this(state, String.Empty, null)
        { }

        public EntityResult(ResultType state, string message, object data = null)
        {
            this.state = state;
            this.message = message;
            this.data = data;
        }

        /// <summary>
        /// 结果类型。
        /// </summary>
        public ResultType state { get; set; }
        /// <summary>
        /// 消息内容。
        /// </summary>
        public string message { get; set; }
        /// <summary>
        /// 返回数据。
        /// </summary>
        public object data { get; set; }
    }


    /// <summary>
    /// 结果类型枚举。
    /// </summary>
    public enum ResultType
    {
        /// <summary>
        /// 异常。
        /// </summary>
        Error = 0,
        /// <summary>
        /// 成功。
        /// </summary>
        Success = 1,
        /// <summary>
        /// Token异常。
        /// </summary>
        TokenError = 2,
        /// <summary>
        /// 服务异常。
        /// </summary>
        ServiceError = 3,
        /// <summary>
        /// 操作失败
        /// </summary>
        Fail = 4

    }
}
