﻿using SuValue.BLL;
using SuValue.Entity;
using SuValue.Entity.Enum;
using SuValue.Entity.Request;
using SuValue.Entity.Responese;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;

namespace Suvalue.Evaluate.Controllers
{
    public class DiseaseController : BaseController
    {
        /// <summary>
        /// 获取疾病RW分类
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Disease/RW")]
        public EntityResult RW([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = " SELECT case when rw >= 5 then 'rw≥5' when rw >= 2 and rw < 5  then '2≤rw<5' when rw < 2  then 'rw<2' end name, count(*) value, sum(person_times) x FROM da_basic_info_{year} {w}  group by  case when rw >= 5 then 'rw≥5' when rw >= 2 and rw < 5  then '2≤rw<5' when rw < 2  then 'rw<2' end;";

                return new EntityResult(ResultType.Success, "", report.GetChartData(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }


        /// <summary>
        /// 获取疾病RW散点报表
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Disease/ReportRW")]
        public EntityResult ReportRW([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL reportbll = new ReportBLL();


                string sql = "SELECT concat(ifnull(disease,''),'&',ifnull(ops,'')) name, rw x , sum(person_times) y  ,sum(person_times) value FROM da_basic_info_{year} {w}  group by  disease,ops order by x desc ,y desc LIMIT 0, 100 ;";

                EntityReport report = new EntityReport() { title = "疾病组合分布", chartType = (int)ChartType.普通散点图, xUnit = "rw", yUnit = "例数", chartData = reportbll.GetChartData(model, sql) };


                return new EntityResult(ResultType.Success, "", report);
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 获取疾病RW排名前10
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Disease/SortRW")]
        public EntityResult SortRW([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL reportbll = new ReportBLL();


                string sql = " SELECT concat(ifnull(disease,''),'&',ifnull(ops,'')) name, rw x , sum(person_times) y  FROM da_basic_info_{year} {w}  group by  disease,ops order by x desc ,y desc LIMIT 0, 10 ;";


                return new EntityResult(ResultType.Success, "", reportbll.GetChartData(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 获取疾病排名
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Disease/TableDisList")]
        public EntityResult TableDisList([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "select concat(ifnull(t1.disease,''),'&',ifnull(t1.ops,'')) name ,t1.rw value ,t2.rw lastValue ,convert((t1.rw-t2.rw) *100/t2.rw,decimal(10,2)) ratio,t1.person_times p_value ,t2.person_times p_lastValue,convert((t1.person_times-t2.person_times) *100/t2.person_times,decimal(10,2)) p_ratio from (select disease, ops, sum(rw) / sum(person_times) rw, sum(person_times) person_times from da_basic_info_{year} {w} group by disease,ops order by rw desc LIMIT 0,100 ) t1 left join (select disease, ops, sum(rw) / sum(person_times) rw, sum(person_times) person_times from da_basic_info_{yesteryear} {w} group by disease,ops ) t2 on t1.disease = t2.disease and t1.ops = t2.ops order by t1.rw desc;";

                return new EntityResult(ResultType.Success, "", report.GetTableData<EntityDiseaseTable>(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 疾病信息
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Disease/DiseaseInfo")]
        public EntityResult DiseaseInfo([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "select t1.person_times,convert((t1.person_times-t2.person_times) *100/t2.person_times,decimal(10,2)) person_times_ratio,t1.person_avg, convert((t1.person_avg-t2.person_avg) *100/t2.person_avg,decimal(10,2))person_avg_ratio,t1.HosCount,convert(t1.HosCount *1.0/t3.HosCount,decimal(10,2)) hos_ratio from  (select disease,ops, sum(person_times)person_times,sum(ifnull(drug_fee,0))/sum(ifnull(drug_person_times,0)) person_avg,count(distinct hospitalid) HosCount FROM da_basic_info_cmi_{year} {w}   ) t1  left join (select disease,ops, sum(person_times)person_times,sum(ifnull(drug_fee,0))/sum(ifnull(drug_person_times,0)) person_avg from da_basic_info_cmi_{yesteryear} {w} )t2 on t1.disease = t2.disease and t1.ops = t2.ops left join (select  count(distinct hospitalid) HosCount  from da_basic_info_cmi_{year} ) t3 on 1=1 ;";

                return new EntityResult(ResultType.Success, "", report.GetTableData<EntityDiseaseInfo>(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }


        /// <summary>
        /// 疾病信息
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Disease/DiseaseReport")]
        public EntityResult DiseaseReport([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                model.MenuID = 62;

                return new EntityResult(ResultType.Success, "", report.GetReportData(model));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }
    }
}
