﻿using System;
using System.Web.Http;

namespace Suvalue.Evaluate.Controllers
{

    using SuValue.BLL;
    using SuValue.Entity;
    using SuValue.Entity.Request;
    using SuValue.Entity.Responese;

    public class DrugController : BaseController
    {

        /// <summary>
        /// 获取药物CMI评分数据
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Drug/CMI")]
        public EntityResult CMI([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "SELECT CASE WHEN CMI >= 1.2 THEN 'CMI >= 1.2' WHEN CMI >= 0.8 AND CMI < 1.2 THEN '0.8 <= CMI < 1.2' WHEN CMI < 0.8 THEN 'CMI < 0.8' END NAME, count(hospitalid) VALUE FROM  (SELECT SUM(DRUG_RW) / SUM(drug_person_times) AS CMI,hospitalid FROM da_basic_info_cmi_{year} {w} GROUP BY hospitalid )t GROUP BY NAME ORDER BY CMI DESC;";

                return new EntityResult(ResultType.Success, "", report.GetChartData(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }


        /// <summary>
        /// 获取药物CMI排名前五
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Drug/SortCMI")]
        public EntityResult SortCMI([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "select hosname name,sum(DRUG_RW)/sum(drug_person_times)value from da_basic_info_cmi_{year} {w} GROUP BY hosname ORDER BY value desc limit 1,5; ";

                return new EntityResult(ResultType.Success, "", report.GetChartData(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }


        /// <summary>
        /// 获取药物指数排名前五
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Drug/SortExponent")]
        public EntityResult SortExponent([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "SELECT hosname name  ,sum(drug_fee)/sum(drug_rw) value FROM da_basic_info_{year} {w} group by hosname order by value desc limit 1,5;";

                return new EntityResult(ResultType.Success, "", report.GetChartData(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }


        /// <summary>
        /// 获取医院CMI汽包报表
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Drug/CMIList")]
        public EntityResult CMIList([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "select t1.name,t2.value x,t1.value y,1 VALUE from (select hosname name,sum(drug_rw) / sum(drug_person_times) value from da_basic_info_cmi_{year} {w} GROUP BY hosname ) t1 join (SELECT hosname name, sum(drug_fee) / sum(drug_RW) value FROM da_basic_info_{year} {w} group by hosname)t2 on t1.name = t2.name; ";

                return new EntityResult(ResultType.Success, "", report.GetChartData(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 获取药物CMI排名
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Drug/TableCMIList")]
        public EntityResult TableCMIList([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "select t1.name title,t1.value  ,t2.value lastValue,  convert((t1.value-t2.value) *100/t2.value,decimal(10,2)) ratio from (select HosName name, SUM(drug_fee) / SUM(drug_rw) value from da_basic_info_{year} {w} GROUP BY HosName ) t1 join(select HosName name, SUM(drug_fee) / SUM(drug_rw) value from da_basic_info_{yesteryear} {w} GROUP BY HosName ) t2 on t1.name = t2.name order by t1.value desc;";

                return new EntityResult(ResultType.Success, "", report.GetTableData<EntityTable>(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

        /// <summary>
        /// 获取药物指数排名
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("api/Drug/TableExponentList")]
        public EntityResult TableExponentList([FromUri]EntitySearch model)
        {
            try
            {
                if (!Authentication(model.token)) return new EntityResult(ResultType.TokenError);

                ReportBLL report = new ReportBLL();

                string sql = "select t1.name title,t1.value  ,t2.value lastValue,  convert((t1.value-t2.value) *100/t2.value,decimal(10,2)) ratio from (select HosName name, SUM(drug_RW) / SUM(drug_person_times) value from da_basic_info_cmi_{year} {w} GROUP BY HosName ) t1 join(select HosName name, SUM(drug_RW) / SUM(drug_person_times)  value from da_basic_info_cmi_{yesteryear} {w} GROUP BY HosName ) t2 on t1.name = t2.name order by t1.value desc;";

                return new EntityResult(ResultType.Success, "", report.GetTableData<EntityTable>(model, sql));
            }
            catch (Exception ex)
            {
                return new EntityResult(ResultType.ServiceError, ex.ToString());
            }
        }

    }
}